#include "idlemgr.hh"
#include <typeinfo>
#include "demo.hh"
#include <stdarg.h>

void demowin::Initialize()
{
  	for(int i=0; i<1000; i++)
  	{
  		double x,y,z;
  		do {
  			x = 1000.0*rand()/(double)RAND_MAX - 500.0;
  			y = 1000.0*rand()/(double)RAND_MAX - 500.0;
  			z = 1000.0*rand()/(double)RAND_MAX - 500.0;
  		} while(sqrt(x*x+y*y+z*z) < 300.0);
  		
  		World.Insert(new line(x,y,z, i%10 ? 1 : 2));
  	}	

  	World.AddPoly( 0, 0, 30, 0, 40,10, 40,30, 30,40,
  				  40,50, 40,70, 30,80,  0,80,  0,00);
  	World.AddPoly(10,10, 30,10, 30,30, 10,30, 10,10);
  	World.AddPoly(10,50, 30,50, 30,70, 10,70, 10,50);
  	World.AddPoly(60,20, 70,20, 65,10, 60,20);
  	World.AddPoly(50,40, 70,30, 70,70, 80,70, 80,80,
  				  50,80, 50,70, 60,70, 60,40, 50,40);
  	World.AddPoly(100,30, 120,30, 130,40, 120,50, 120,40,
  				  100,40, 100,50, 130,60, 130,70, 120,80,
  				  100,80,  90,70, 100,60, 100,70, 120,70,
  				  120,60,  90,50,  90,40, 100,30);
  	World.AddPoly(150,30, 180,30, 180,110,140,70, 140,40,
  	              150,30);
  	World.AddPoly(150,40, 170,40, 170,70, 150,70, 150,40);
  	World.AddPoly(190,30, 200,50, 220,50, 230,30, 230,80,
  				  210,60, 190,80, 190,30);
  	World.AddPoly(240,40, 260,30, 260,70, 270,70, 270,80,
  				  240,80, 240,70, 250,70, 250,40, 240,40);
  	World.AddPoly(250,20, 260,20, 255,10, 250,20);
  	World.AddPoly(290,20, 300,20, 300,30, 310,30, 310,40,
  				  300,40, 310,80, 290,80, 290,40, 280,40,
  				  280,30, 290,30, 290,20);
  	
  	ax=-0.005*60;
  	ay=-0.008*60;
  	az=-0.009*60;
  	cx=cy=0.0;
  	cz=-20;
  	
    idlemgr.registertask(this, 10);
}
void demowin::Init()
{
	EC=width()&~1, EL=height(), EC2=EC*2, EL3=EL*3;
	Field.resize(EC*EL3); memset(&Field[0], 0, Field.size() * sizeof(Field[0]));
	Color.resize(EC*EL);
	efi.resize(EC*EL);
}
void demowin::Done()
{
	if(EC==width() && EL==height())
	{
		/*  ab                                  */
		/*  cd                       d          */
		/*  ef         c      d    ddc          */
		/*      b    ccb    ddb   dccb          */
		/*    aba   caba   daba   caba          */
		char Buf[] =
			" ~~\"""-'^*" "-^'*""-\\/\""  /*    */
			".;;P" ",|/P" ",>/?" "r}/P"   /* e  */
			".::M"",\\<t"",\\|7""q\\q9"   /* f  */
			"__J=" "_[ZC" ",SJ]" "gbd@";  /* ef */
			
		/*	ABCDEFGHIJKLMNOPQRSTUVWXYZ */
		/*  abcdefghijklmnopqrstuvwxyz */
		/*   !"#$%&'()*+,-./0123456789 */
		/*  :;<=>?@[\]^_{|}~           */
		
		int x,y;
		const char *s = &Field[0];
		const char *c = &Color[0];
		char *e = &efi[0];
		
		for(y=3; y<EL; y++, s+=EC2)
			for(x=0; x<EC; ++x,  ++s,++e,++c)
			{
				int ch = Buf[(s[0  ]  )
				           + (s[EC ]<<2)
				           + (s[EC2]<<4)];
				           
				if(*e != ch)
				{
					buf.buf[y][x].attr = "\6\1\11"[*c];
					buf.buf[y][x].ch = (*e=ch);
				}
			}
	}
}
void demowin::Line(double X1,double Y1, double X2,double Y2)
{
	int x1,y1, x2,y2;
	
	x1=int(X1*(EC2));
	x2=int(X2*(EC2));
	y1=int(Y1*(EL3));
	y2=int(Y2*(EL3));
	
	int	i, DeltaX, DeltaY, NumPixels,
		d, dinc1, dinc2,
		x, xinc1, xinc2,
		y, yinc1, yinc2;

	/* Calculate DeltaX and DeltaY for init */
	DeltaX = abs(x2 - x1);
	DeltaY = abs(y2 - y1);

	/* Initialize all vars based on which is the independent variable k1 */
	if(DeltaX >= DeltaY)
	{
		/* x is independent variable */
		NumPixels = DeltaX + 1;
		d = (DeltaY << 1) - DeltaX;
    	dinc1 = DeltaY << 1;
		dinc2 = (DeltaY - DeltaX) << 1;
		yinc1 = 0, xinc1 = xinc2 = yinc2 = 1;
	}
	else
	{
		/* y is independent variable */
		NumPixels = DeltaY + 1;
		d = (DeltaX << 1) - DeltaY;
    	dinc1 = DeltaX << 1;
		dinc2 = (DeltaX - DeltaY) << 1;
		xinc1 = 0, yinc1 = yinc2 = xinc2 = 1;
	}

	/* Make sure x and y move in the right directions */
	if(x1 > x2) {xinc1 = -xinc1; xinc2 = -xinc2;}
	if(y1 > y2) {yinc1 = -yinc1; yinc2 = -yinc2;}
	
	/* Start drawing at <x1, y1> */
  	x = x1; y = y1;
  	
  	/* Draw the pixels */
	for(i=1; i<NumPixels; i++)
	{
		if(SetP(x, y))Color[EC * (y/3) + (x>>1)] = 0;
			
		if(d<0)	d += dinc1, x += xinc1, y += yinc1;
		else	d += dinc2, x += xinc2, y += yinc2;
	}
}
void demowin::handleevent(event &e)
{
	window::handleevent(e);
	if(typeid(*e) == typeid(event_idle))
	{
   		Init();
   		
   		World::iterator i;
   		
   		Matrix m(ax,ay,az);
   		
   		for(i=World.begin(); i<World.end(); i++)
   			(*i)->Draw(this, cx,cy,cz, m);
   		
   		ax += 0.005;
   		ay += 0.008;
   		az += 0.009;
   		
   		Done();
	}
}
demowin::~demowin()
{
	idlemgr.unregistertask(this);
}
void demowin::line::Draw(demowin *w, double cx,double cy,double cz, Matrix &m)
{
	if(isline)
	{
		double X1=x1, Y1=y1, Z1=z1;
		double X2=x2, Y2=y2, Z2=z2;
		
		m.RotatePoint(X1,Y1,Z1);
		m.RotatePoint(X2,Y2,Z2);
		
		X1-=cx; Y1-=cy; Z1-=cz;
		X2-=cx; Y2-=cy; Z2-=cz;
		
		if(Z1 > 0 && Z2 > 0)
			w->Line(0.5+X1/Z1, 0.5+Y1/Z1,
			        0.5+X2/Z2, 0.5+Y2/Z2);
	}
	else
	{
		double x = x1, y = y1, z = z1;
		m.RotatePoint(x,y,z);
		x-=cx, y-=cy, z-=cz;
		if(z > 0)
			w->PSet(0.5+x/z, 0.5+y/z, color);
	}
}
void demowin::World::AddPoly(int x,int y, ...)
{
	va_list ap;
	va_start(ap, y);
	
	double x1=x/10.0, y1=y/10.0;
	
	for(;;)
	{
		int X = va_arg(ap, int);
		int Y = va_arg(ap, int);
		double x2 = X / 10.0;
		double y2 = Y / 10.0;
		
		Insert(new line(x1,y1,0, x2,y2,0));
		Insert(new line(x1,y1,1, x2,y2,1));
		
		Insert(new line(x1,y1,0, x1,y1,1));
		Insert(new line(x2,y2,0, x2,y2,1));
		
		if(X==x && Y==y)break;
		x1=x2, y1=y2;
	}
	va_end(ap);
}
