#ifndef bqt_window_hh
#define bqt_window_hh

#include <list>

#include "winbuf.hh"
#include "autoptr"
#include "events.hh"
#include "rect.hh"

typedef autoptr<class window> winptr;
class window : public ptrable
{
private:
	class childwin
	{
	public:
		winptr p;
		int x, y;
	public:
		explicit childwin(winptr w) : p(w), x(0), y(0) { }
		childwin(winptr w, int xp, int yp) : p(w), x(xp), y(yp) { }
	};
	typedef list<childwin> childlist;
	childlist children;
	winptr owner;
	bool focused;
private:
	void redraw_after_move(const rect &oldr, const rect &newr);
	childlist::iterator find_child(const window *p);
	childlist::const_iterator find_child(const window *p) const;
	unsigned getchildxpos(const window *p) const { childlist::const_iterator i = find_child(p); return i->x; }
	unsigned getchildypos(const window *p) const { childlist::const_iterator i = find_child(p); return i->y; }
	void movechild(const window *p, int x, int y)
	{
		childlist::iterator i = find_child(p);
		/* printf("Resized child %p: (%u,%u) (location: %u,%u)\n", p, x,y, p->width(),p->height()); */
		i->x = x;
		i->y = y;
	}
	void killchild(window *p)
	{
		childlist::iterator i = find_child(p);
		if(i != children.end())children.erase(i);
	}
public:
	winbuf buf;
	window() : focused(true)
	{
	}
	window(unsigned x, unsigned y) : focused(true), buf(x, y)
	{
	}
	virtual ~window()
	{
		if(owner) /* The root window doesn't have owner */
		{
			winptr ow = owner;
			owner = NULL;
			ow->killchild(this);
		}
	}
	
	void drawchildren() { drawchildren(0,0, width()-1, height()-1); }
	virtual void drawchildren(int x1,int y1, int x2,int y2);

	/* No window is forced to redraw anything unless explicitly called
	 * by this routine. Even then, the window can ignore it, as the 
	 * buffer already may contain sufficient data. This is just a
	 * way to tell the window that now this region of buffer will
	 * be displayed on the screen, and it would be nice to have
	 * something sensible there.
	 */
	virtual void redraw(int x1,int y1, int x2,int y2);
	void redraw(const rect &r) { if(r)redraw(r.x1,r.y1,r.x2,r.y2); }
	virtual void redraw() { redraw(0,0, width()-1,height()-1); }
	inline unsigned width() const { return buf.width(); }
	inline unsigned height() const { return buf.height(); }
	inline int getxpos() const { return owner?owner->getchildxpos(this):0; }
	inline int getypos() const { return owner?owner->getchildypos(this):0; }
	virtual void resize(unsigned nxs, unsigned nys);
	virtual void move(int x, int y); /* parameters: new position within owner */
	inline void addchild(winptr p) { addchild(p, 0, 0); }
	void addchild(winptr p, int x, int y);
	
	virtual void handleevent(event &e);
};

#endif
